use super::{order, priority, IncomingPriority, Never, OutgoingPriority, Priority, Service, API};
use crate::io::Packet;

use crate as fleck;

crate::service_priority!(
    SendPacket,
    out: Priority<order::Postprocessing, order::Last>
);

#[derive(Default)]
pub(crate) struct SendPacket {}

impl Service for SendPacket {
    fn process_outgoing(&mut self, api: &dyn API, packet: &mut crate::io::Packet) {
        // use default channel if not specified
        if packet.channel.is_none() {
            packet.channel = Some(api.raw_io().global().clone());
        }

        // serialize if needed
        match (packet.data.as_ref().is_some(), packet.msg.take()) {
            (false, Some(msg)) => {
                packet.data = Some(bincode::serialize(&msg).expect("failed to serialize message"));
            }
            _ => {}
        }

        match &mut packet.channel {
            Some(ch) => ch.clone().send_packet(packet),
            _ => {
                println!("tried to send packet with no channel?");
            }
        }
    }
}

crate::service_priority!(LocalDiscovery);
pub(crate) struct LocalDiscovery {}

impl LocalDiscovery {
    pub(crate) fn new() -> Self {
        Self {}
    }
}

impl Service for LocalDiscovery {
    fn process_major_tick(&mut self, api: &dyn API) {
        api.send_packet(Packet {
            addr: Some(crate::io::MULTICAST_ADDRESS.clone().into()),
            data: None,
            channel: Some(api.raw_io().local().clone()),
            msg: Some(crate::msg::Message::build(crate::msg::HeaderType::Hello)),
        });
    }
}
